@@ -30,6 +30,7 @@
 #include <KStandardGuiItem>
 
 #include <QElapsedTimer>
+#include <QRandomGenerator>
 
 #include <cstdio>
 #include <ctime>
@@ -57,17 +58,35 @@ SudokuBoard::SudokuBoard (SKGraph * graph)
                         << ", BoardArea " << m_boardArea;
 }
 
+#define SEEDED_RANDROMGENERATOR 0
+
+static QRandomGenerator* randomGenerator()
+{
+#if SEEDED_RANDROMGENERATOR
+    static QRandomGenerator ourRandomGenerator;
+    return &ourRandomGenerator;
+#else
+    return QRandomGenerator::global();
+#endif
+}
+
 void SudokuBoard::setSeed()
 {
     static bool started = false;
     if (started) {
+#if SEEDED_RANDROMGENERATOR
+        qCDebug(KSudokuLog) << "setSeed(): RESET IS TURNED ON";
+        randomGenerator()->seed(m_stats.seed); // IDW test.
+#else
         qCDebug(KSudokuLog) << "setSeed(): RESET IS TURNED OFF";
-        // qsrand (m_stats.seed); // IDW test.
+#endif
     }
     else {
         started = true;
         m_stats.seed = std::time(nullptr);
-        qsrand (m_stats.seed);
+#if SEEDED_RANDROMGENERATOR
+        randomGenerator()->seed(m_stats.seed);
+#endif
         qCDebug(KSudokuLog) << "setSeed(): SEED = " << m_stats.seed;
     }
 }
@@ -100,11 +119,12 @@ bool SudokuBoard::generatePuzzle             (BoardContents & puzzle,
 	    }
 	    else if (numTries >= maxTries) {
 		QWidget owner;
-		if (KMessageBox::questionYesNo (&owner,
+		if (KMessageBox::questionTwoActions (&owner,
 			    i18n("Attempts to generate a puzzle failed after "
 				 "about 200 tries. Try again?"),
-			    i18n("Mathdoku or Killer Sudoku Puzzle"))
-			    == KMessageBox::No) {
+			    i18n("Mathdoku or Killer Sudoku Puzzle"),
+			    KGuiItem(i18n("&Try Again")), KStandardGuiItem::cancel())
+			    == KMessageBox::SecondaryAction) {
 		    return false;	// Go back to the Welcome screen.
 		}
 		numTries = 0;		// Try again.
@@ -139,12 +159,12 @@ bool SudokuBoard::generateSudokuRoxdokuTypes (BoardContents & puzzle,
 	symmetry = NONE;		// Symmetry not implemented in 3-D.
     }
     if (symmetry == RANDOM_SYM) {	// Choose a symmetry at random.
-        symmetry = (Symmetry) (qrand() % (int) LAST_CHOICE);
+        symmetry = (Symmetry) (randomGenerator()->bounded((int) LAST_CHOICE));
     }
     qCDebug(KSudokuLog) << "SYMMETRY IS" << symmetry;
     if (symmetry == DIAGONAL_1) {
 	// If diagonal symmetry, choose between NW->SE and NE->SW diagonals.
-        symmetry = (qrand() % 2 == 0) ? DIAGONAL_1 : DIAGONAL_2;
+        symmetry = (randomGenerator()->bounded(2) == 0) ? DIAGONAL_1 : DIAGONAL_2;
         qCDebug(KSudokuLog) << "Diagonal symmetry, choosing " <<
             ((symmetry == DIAGONAL_1) ? "DIAGONAL_1" : "DIAGONAL_2");
     }
@@ -193,7 +213,7 @@ bool SudokuBoard::generateSudokuRoxdokuTypes (BoardContents & puzzle,
 	if ((d < difficultyRequired) && (count >= maxTries)) {
             // Exit after max attempts?
             QWidget owner;
-            int ans = KMessageBox::questionYesNo (&owner,
+            int ans = KMessageBox::questionTwoActions (&owner,
                       i18n("After %1 tries, the best difficulty level achieved by the generator "
 			   "is %2, with internal difficulty rating %3, but you "
 			   "requested difficulty level %4.\n"
@@ -207,7 +227,7 @@ bool SudokuBoard::generateSudokuRoxdokuTypes (BoardContents & puzzle,
 			   ratingStr, difficultyRequired),
                       i18n("Difficulty Level"),
                       KGuiItem(i18n("&Try Again")), KGuiItem(i18n("&Accept")));
-            if (ans == KMessageBox::Yes) {
+            if (ans == KMessageBox::PrimaryAction) {
                 count = 0;	// Continue on if the puzzle is not hard enough.
                 continue;
             }
@@ -217,7 +237,7 @@ bool SudokuBoard::generateSudokuRoxdokuTypes (BoardContents & puzzle,
             QWidget owner;
 	    int ans = 0;
 	    if (m_accum.nGuesses == 0) {
-                ans = KMessageBox::questionYesNo (&owner,
+                ans = KMessageBox::questionTwoActions (&owner,
 		       i18n("It will be possible to solve the generated puzzle "
 			    "by logic alone. No guessing will be required.\n"
 			    "\n"
@@ -231,7 +251,7 @@ bool SudokuBoard::generateSudokuRoxdokuTypes (BoardContents & puzzle,
 	    else {
                 QString avGuessStr = ki18n("%1").subs(((float) bestNGuesses) /
 			5.0, 0, 'f', 1).toString(); // Format as for ratingStr.
-                ans = KMessageBox::questionYesNo (&owner,
+                ans = KMessageBox::questionTwoActions (&owner,
 		       i18n("Solving the generated puzzle will require an "
 			    "average of %1 guesses or branch points and if you "
 			    "guess wrong, backtracking will be necessary. The "
@@ -245,7 +265,7 @@ bool SudokuBoard::generateSudokuRoxdokuTypes (BoardContents & puzzle,
                        KStandardGuiItem::ok(), KGuiItem(i18n("&Retry")));
 	    }
 	    // Exit when the required difficulty or number of tries is reached.
-            if (ans == KMessageBox::No) {
+            if (ans == KMessageBox::SecondaryAction) {
                 count = 0;
                 bestRating = 0.0;
                 bestDifficulty = 0;
@@ -395,7 +415,7 @@ BoardContents & SudokuBoard::tryGuesses (GuessingMode gMode = Random)
         GuessesList guesses = m_states.top()->guesses();
         int n = m_states.top()->guessNumber();
         if ((n >= guesses.count()) || (guesses.at (0) == -1)) {
-            dbo2 "POP: Out of guesses at level %d\n", m_states.count());
+            dbo2 "POP: Out of guesses at level %" PRIdQSIZETYPE "\n", m_states.count());
             delete m_states.pop();
             if (!m_states.isEmpty()) {
                 m_moves.clear();
@@ -410,7 +430,7 @@ BoardContents & SudokuBoard::tryGuesses (GuessingMode gMode = Random)
         m_moves.append (guesses.at(n));
         m_moveTypes.append (Guess);
         m_currentValues [pairPos (guesses.at(n))] = pairVal (guesses.at(n));
-        dbo2 "\nNEXT GUESS: level %d, guess number %d\n",
+        dbo2 "\nNEXT GUESS: level %" PRIdQSIZETYPE ", guess number %d\n",
                 m_states.count(), n);
         dbo2 "  Pick %d %d row %d col %d\n",
                 pairVal (guesses.at(n)), pairPos (guesses.at(n)),
@@ -440,7 +460,7 @@ BoardContents SudokuBoard::insertValues (const BoardContents & solution,
 {
     BoardContents puzzle;
     BoardContents filled;
-    QVector<int> sequence (m_boardArea);
+    QList<int> sequence (m_boardArea);
     int cell  = 0;
     int value = 0;
 
@@ -449,14 +469,14 @@ BoardContents SudokuBoard::insertValues (const BoardContents & solution,
     clear (filled);
 
     // Add cells in random order, but skip cells that can be deduced from them.
-    dbo1 "Start INSERTING: %d solution values\n", solution.count());
+    dbo1 "Start INSERTING: %" PRIdQSIZETYPE " solution values\n", solution.count());
     randomSequence (sequence);
 
     int index = 0;
     for (int n = 0; n < m_boardArea; n++) {
         cell  = sequence.at (n);
         value = filled.at (cell);
-        if (filled.at (cell) == 0) {
+        if (value == 0) {
             index = n;
             changeClues (puzzle, cell, symmetry, solution);
             changeClues (filled, cell, symmetry, solution);
@@ -505,7 +525,7 @@ BoardContents SudokuBoard::removeValues (const BoardContents & solution,
 
     // Remove values in random order, but put them back if the solution fails.
     BoardContents vacant;
-    QVector<int> sequence (m_boardArea);
+    QList<int> sequence (m_boardArea);
     int          cell       = 0;
     int          value      = 0;
     QList<int>   tailOfRemoved;
@@ -556,7 +576,7 @@ BoardContents SudokuBoard::removeValues (const BoardContents & solution,
             if (m_stats.difficulty == required) {
                 // Save removed positions while the difficulty is as required.
                 tailOfRemoved.append (cell);
-                dbo1 "OVERSHOOT %d at sequence %d\n",
+                dbo1 "OVERSHOOT %" PRIdQSIZETYPE " at sequence %d\n",
                         tailOfRemoved.count(), n);
             }
 
@@ -691,7 +711,7 @@ void SudokuBoard::print (const BoardContents & boardValues)
     int index, value;
 
     if (boardValues.size() != m_boardArea) {
-        printf ("Error: %d board values to be printed, %d values required.\n\n",
+        printf ("Error: %" PRIdQSIZETYPE " board values to be printed, %d values required.\n\n",
             boardValues.size(), m_boardArea);
         return;
     }
@@ -782,7 +802,7 @@ GuessesList SudokuBoard::deduceValues (BoardContents & boardValues,
                         ;
                     }
                     else if (gMode == Random) {
-                        if ((qrand() % count) == 0) {
+                        if (randomGenerator()->bounded(count) == 0) {
                             guesses = newGuesses;
                         }
                         count++;
@@ -792,7 +812,7 @@ GuessesList SudokuBoard::deduceValues (BoardContents & boardValues,
         } // Next cell
 
         for (int group = 0; group < m_nGroups; group++) {
-	    QVector<int> cellList = m_graph->clique (group);
+	    QList<int> cellList = m_graph->clique (group);
             qint32 numbers = m_requiredGroupValues.at (group);
             dbo3 "Group %d, valid numbers %03o\n", group, numbers);
             if (numbers == 0) {
@@ -839,7 +859,7 @@ GuessesList SudokuBoard::deduceValues (BoardContents & boardValues,
                             ;
                         }
                         else if (gMode == Random){
-                            if ((qrand() % count) == 0) {
+                            if (randomGenerator()->bounded(count) == 0) {
                                 guesses = newGuesses;
                             }
                             count++;
@@ -856,7 +876,7 @@ GuessesList SudokuBoard::deduceValues (BoardContents & boardValues,
             GuessesList original = guesses;
             if (gMode == Random) {
                 // Shuffle the guesses.
-                QVector<int> sequence (guesses.count());
+                QList<int> sequence (guesses.count());
                 randomSequence (sequence);
 
                 guesses.clear();
@@ -903,8 +923,8 @@ BoardContents & SudokuBoard::fillBoard()
 
     // Fill a central block with values 1 to m_order in random sequence.  This
     // reduces the solveBoard() time considerably, esp. if blockSize is 4 or 5.
-    QVector<int> sequence (m_order);
-    QVector<int> cellList = m_graph->clique (m_nGroups / 2);
+    QList<int> sequence (m_order);
+    QList<int> cellList = m_graph->clique (m_nGroups / 2);
     randomSequence (sequence);
     for (int n = 0; n < m_order; n++) {
         m_currentValues [cellList.at (n)] = sequence.at (n) + 1;
@@ -915,7 +935,7 @@ BoardContents & SudokuBoard::fillBoard()
     return m_currentValues;
 }
 
-void SudokuBoard::randomSequence (QVector<int> & sequence)
+void SudokuBoard::randomSequence (QList<int> & sequence)
 {
     if (sequence.isEmpty()) return;
 
@@ -932,7 +952,7 @@ void SudokuBoard::randomSequence (QVector<int> & sequence)
     int z    = 0;
     int temp = 0;
     for (int i = 0; i < size; i++) {
-        z = qrand() % last;
+        z = randomGenerator()->bounded(last);
         last--;
         temp            = sequence.at (z);
         sequence [z]    = sequence.at (last);
@@ -960,7 +980,7 @@ void SudokuBoard::setUpValueRequirements (BoardContents & boardValues)
     qint32 bitPattern = 0;
     for (int group = 0; group < m_nGroups; group++) {
 	dbo3 "Group %3d ", group);
-	QVector<int> cellList = m_graph->clique (group);
+	QList<int> cellList = m_graph->clique (group);
         bitPattern = 0;
         for (int n = 0; n < m_groupSize; n++) {
             int value = boardValues.at (cellList.at (n)) - 1;
@@ -995,7 +1015,7 @@ void SudokuBoard::setUpValueRequirements (BoardContents & boardValues)
     // can only have 7 or 8, with bit value 192.
     index = 0;
     for (int group = 0; group < m_nGroups; group++) {
-	QVector<int> cellList = m_graph->clique (group);
+	QList<int> cellList = m_graph->clique (group);
         for (int n = 0; n < m_order; n++) {
 	    int cell = cellList.at (n);
             m_validCellValues [cell] &= m_requiredGroupValues.at (group);
@@ -1031,7 +1051,7 @@ void SudokuBoard::updateValueRequirements (BoardContents & boardValues, int cell
     for (int group : groupList) {
         m_requiredGroupValues [group] &= bitPattern;
 
-	QVector<int> cellList = m_graph->clique (group);
+	QList<int> cellList = m_graph->clique (group);
         for (int n = 0; n < m_order; n++) {
 	    int cell = cellList.at (n);
             m_validCellValues [cell] &= bitPattern;
@@ -1072,6 +1092,7 @@ int SudokuBoard::getSymmetricIndices
             row = tb;
             col = lr;
             // No break; fall through to case DIAGONAL_2.
+            [[fallthrough]];
         case DIAGONAL_2:
 	    // Reflect (col, row) in the main NW-SE diagonal by swapping coords.
             out[1] = m_graph->cellIndex(row, col);
